particle_types = require ("particle-rules")

screen = {
		w = love.graphics.getWidth(),
		h = love.graphics.getHeight()
	}

--G = 10^6
G = 2*10^5
amount = 27*10
speed_max = 150
min_dist = 25
--brake = 10^(-1)
brake = 1/2
game_started = true

function get_color (particle_type)
	local r = 0.5+0.5*(particle_type[2]+1)
	local g = 0.5+0.5*(particle_type[3]+1)
	local b = 0.5+0.5*(particle_type[4]+1)
	local a = 1
	return {r,g,b,a}
end


function get_size (particle_type)
	local sizes = {a = 4, b = 3, c = 2}
	local size = sizes[particle_type[1]]
	return size
end


function particle_types_interaction (p1, p2)
	local interaction = 0
	if p2.particle_type[1] == "a" then
		interaction = p2.particle_type[2]
	elseif p2.particle_type[1] == "b" then
		interaction = p2.particle_type[3]
	elseif p2.particle_type[1] == "c" then
		interaction = p2.particle_type[4]
	end
	return interaction
end


function love.load()
	love.math.setRandomSeed(love.timer.getTime())

	particles = {}
	interactions = {}
	for id_particle = 1, amount do
--		local particle_type = particle_types[math.random (#particle_types)]
		local particle_type = particle_types[(1+id_particle%#particle_types)]
		local color = get_color (particle_type)
		local size = get_size (particle_type)
		local x = math.random(size+10, screen.w-size)
		local y = math.random(size+10, screen.h-size)
		particles[id_particle] = {x=x, y=y, size = size, speed={x=0,y=0}, 
			force={x=0,y=0}, last_force={x=0,y=0}, 
			color = color, particle_type = particle_type}
	end
	print ('particles: ' .. #particles)
	
	for id_p1 = 1, (amount-1) do
		for id_p2 = (id_p1+1), amount do
			local p1 = particles[id_p1]
			local p2 = particles[id_p2]
			if p1 and p2 then
				local int1 = particle_types_interaction (p1, p2)
				local int2 = particle_types_interaction (p2, p1)
				interactions[#interactions+1] = {id_p1, id_p2, int1, int2}
			else
				print ('no interaction: ' .. id_p1..'-'..id_p2)
			end
		end
	end
	print ('interactions: ' .. #interactions)
end

function update_force (interaction, dt)
	local p1 = particles[interaction[1]]
	local p2 = particles[interaction[2]]
	local int1 = interaction[3]
	local int2 = interaction[4]
	local dx = p1.x-p2.x
	local dy = p1.y-p2.y
	local sd = (dx^2 + dy^2) -- distance is already square
	local s = math.sqrt(sd) -- distance
	if s > min_dist then
		local alpha = math.atan2(dy, dx)
		local m1 = 1
		local m2 = 1
	--	local force = G*m1*m2/sd
	--	print (dx)
		local f_x = G*m1*m2/(sd) * math.cos(alpha)
		local f_y = G*m1*m2/(sd) * math.sin(alpha)
		
		p1.force.x = p1.force.x - f_x * int1
		p1.force.y = p1.force.y - f_y * int1
		
		p2.force.x = p2.force.x + f_x * int2
		p2.force.y = p2.force.y + f_y * int2
	elseif s < (0.9*min_dist) then
		local alpha = math.atan2(dy, dx)
		local m1 = 1
		local m2 = 1
--		local force = (math.sqrt(sd)/(min_dist^2)) *G*m1*m2/(min_dist^2)
--		local force = (G*m1*m2/(min_dist^2))^(sd)
		local force = G*m1*m2/(min_dist^2)
		force = force * 10* ((min_dist-s)/min_dist)
		local f_x = force * math.cos(alpha)
		local f_y = force * math.sin(alpha)
		
		p1.force.x = p1.force.x + f_x* math.abs(int1)
		p1.force.y = p1.force.y + f_y* math.abs(int1)
		
		p2.force.x = p2.force.x - f_x* math.abs(int2)
		p2.force.y = p2.force.y - f_y* math.abs(int2)
	end
end

function update_speed (particle, dt)
	local vx = particle.speed.x + dt*particle.force.x
	local vy = particle.speed.y + dt*particle.force.y
	local speed = math.sqrt(vx^2 + vy^2)
	if speed > speed_max then
		vx = vx * (speed_max/speed)
		vy = vy * (speed_max/speed)
	else
		local alpha = math.atan2(vy, vx)
		vx = vx - brake * math.cos(alpha)
		vy = vy - brake * math.sin(alpha)
	end
	particle.speed.x = vx
	particle.speed.y = vy
	particle.last_force = {x=particle.force.x, y=particle.force.y}
	particle.force = {x=0,y=0}
--	print (particle.speed.x)
end

function update_position (particle, dt)
	
	if enabled_limits then
		if particle.x < (particle.size) then 
			particle.speed.x = math.abs(particle.speed.x )/2
		elseif particle.x > (screen.w-particle.size) then
			particle.speed.x = -math.abs(particle.speed.x )/2
		end
		if particle.y < (particle.size) then 
			particle.speed.y = math.abs(particle.speed.y)/2
		elseif particle.y > (screen.h-particle.size) then
			particle.speed.y = -math.abs(particle.speed.y)/2
		end
	else
		if particle.x < 0 then 
			particle.x = screen.w+particle.x
		elseif particle.x > screen.w then
			particle.x = particle.x-screen.w
		end
		if particle.y < (particle.size) then 
			particle.speed.y = math.abs(particle.speed.y)/2
		elseif particle.y > (screen.h-particle.size) then
			particle.y = (screen.h-particle.size)
			particle.speed.y = -math.abs(particle.speed.y)/2
		end
	end
	particle.x = particle.x + dt*particle.speed.x
	particle.y = particle.y + dt*particle.speed.y
end

function love.update(dt)
	if game_started then
		for id_interaction, interaction in pairs (interactions) do
			update_force (interaction, dt)
		end
		
		for id_particle, particle in pairs (particles) do
			update_speed (particle, dt)
			update_position (particle, dt)
		end
	end
end


function love.draw()
--	local mx, my = love.
	for id_particle, particle in pairs (particles) do
		love.graphics.setColor(particle.color)
		if particle.particle_type[1] == "b" then
			love.graphics.circle("fill", particle.x, particle.y, particle.size)
			love.graphics.line(particle.x, particle.y, particle.x-particle.speed.x/10, particle.y-particle.speed.y/10)
			love.graphics.setColor(0,0,0,1)
--			love.graphics.points(particle.x, particle.y)
			love.graphics.circle("fill", particle.x, particle.y, 2)
		else
			love.graphics.circle("fill", particle.x, particle.y, particle.size)
			love.graphics.line(particle.x, particle.y, particle.x-particle.speed.x/10, particle.y-particle.speed.y/10)
			love.graphics.setColor(1,1,1,1)
			love.graphics.points(particle.x, particle.y)
		end

		
		
		-- forces, just disabled
--		love.graphics.setColor(1,0,0,1)
--		local k_force = 5*10^(-2)
--		love.graphics.line(particle.x, particle.y, particle.x+particle.last_force.x*k_force, particle.y+particle.last_force.y*k_force)
	end
	love.graphics.setColor(1,1,1,1)
	love.graphics.print("Current FPS: "..tostring(love.timer.getFPS( )), 10, 10)
	love.graphics.print("Pause: space", 10, 22)
	love.graphics.print("Restart: R", 10, 34)
--	print(particles[1].x)
end

function love.resize(w, h)
--	print(("Window resized to width: %d and height: %d."):format(w, h))
	screen = {
		w = w,
		h = h
	}
end


function restart ()
	game_started = false
	for id_particle, particle in pairs (particles) do
		particle.x = math.random(particle.size+10, screen.w-particle.size)
		particle.y = math.random(particle.size+10, screen.h-particle.size)
		particle.speed.x = 0
		particle.speed.y = 0
	end
end


function love.keypressed(key, unicode)
	if key == "escape" then
		love.event.push("quit")
	end
	if key == "space" then
		game_started = not game_started
	elseif key == "r" then
		restart ()
	end
end